
export interface Results {
    query: string;
    originalQuery: string;
    correctedQuery: string;
    area: string;
    biasingProfile: string;
    redirect: string;
    template: Template;
    pageInfo: PageInfo;
    totalRecordCount: number;
    records: Record[];
    availableNavigation: Navigation[];
    selectedNavigation: Navigation[];
    didYouMean: string[];
    relatedQueries: string[];
    rewrites: string[];
    errors: string;
    warnings: string[];
    debugInfo: DebugInfo;
}
export interface Template {
    name: string;
    ruleName: string;
    zones: any;
}
export interface PageInfo {
    recordStart: number;
    recordEnd: number;
}
export interface DebugInfo {
    rawRequest: any;
    rawResponse: any;
    rawAggregationsRequest: any;
    rawAggregationsResponse: any;
}
export interface Record {
    id: string;
    url: string;
    title: string;
    snippet?: string;
    allMeta: any;
}
export interface Navigation {
    name: string;
    refinements: Array<ValueRefinement | RangeRefinement>;
    ignored?: boolean;
}
export declare type RefinementType = 'Value' | 'Range';
export interface Refinement {
    exclude?: boolean;
    type: RefinementType;
}
export interface ValueRefinement extends Refinement {
    value: string;
}
export interface RangeRefinement extends Refinement {
    low: number;
    high: number;
}

import { Refinement, ValueRefinement, RangeRefinement } from './response';
export declare class Request {
    query: string;
    fields: string[];
    orFields: string[];
    includedNavigations: string[];
    excludedNavigations: string[];
    sort: Sort[];
    customUrlParams: CustomUrlParam[];
    refinements: SelectedRefinement[];
    restrictNavigation: RestrictNavigation;
    biasing: Biasing;
    matchStrategy: MatchStrategy;
    userId: string;
    language: string;
    collection: string;
    area: string;
    biasingProfile: string;
    skip: number;
    pageSize: number;
    returnBinary: boolean;
    pruneRefinements: boolean;
    disableAutocorrection: boolean;
    wildcardSearchEnabled: boolean;
}
export declare type SortOrder = 'Ascending' | 'Descending';
export interface Sort {
    field: string;
    order: SortOrder;
}
export interface CustomUrlParam {
    key: string;
    value: string;
}
export interface SelectedRefinement extends Refinement {
    navigationName: string;
}
export interface SelectedRangeRefinement extends SelectedRefinement, RangeRefinement {
}
export interface SelectedValueRefinement extends SelectedRefinement, ValueRefinement {
}
export interface RestrictNavigation {
    name: string;
    count: number;
}
export declare type BiasStrength = 'Absolute_Increase' | 'Strong_Increase' | 'Medium_Increase' | 'Weak_Increase' | 'Leave_Unchanged' | 'Weak_Decrease' | 'Medium_Decrease' | 'Strong_Decrease' | 'Absolute_Decrease';
export interface Bias {
    name: string;
    content?: string;
    strength: BiasStrength;
}
export interface Biasing {
    bringToTop?: string[];
    augmentBiases: boolean;
    biases: Bias[];
    influence?: number;
}
export interface PartialMatchRule {
    terms?: number;
    termsGreaterThan?: number;
    mustMatch?: number;
    percentage?: boolean;
}
export interface MatchStrategy {
    rules: PartialMatchRule[];
}

import { SelectedRefinement } from '../models/request';
import { Navigation } from '../models/response';
export declare class NavigationConverter {
    static convert(navigations: Array<Navigation>): Array<SelectedRefinement>;
}

import { Request, SelectedValueRefinement, SelectedRangeRefinement, CustomUrlParam, RestrictNavigation, Sort, MatchStrategy, Biasing } from '../models/request';
import { ValueRefinement, RangeRefinement, Navigation } from '../models/response';
export interface QueryConfiguration {
    userId?: string;
    language?: string;
    collection?: string;
    area?: string;
    biasingProfile?: string;
}
export declare class Query {
    private request;
    private unprocessedNavigations;
    queryParams: any;
    constructor(query?: string);
    withQuery(query: string): Query;
    withConfiguration(configuration: QueryConfiguration): Query;
    withSelectedRefinements(...refinements: Array<SelectedValueRefinement | SelectedRangeRefinement>): Query;
    withoutSelectedRefinements(...refinements: Array<SelectedValueRefinement | SelectedRangeRefinement>): Query;
    withRefinements(navigationName: string, ...refinements: Array<ValueRefinement | RangeRefinement>): Query;
    withNavigations(...navigations: Navigation[]): Query;
    withCustomUrlParams(customUrlParams: CustomUrlParam[] | string): Query;
    private convertParamString(customUrlParams);
    withFields(...fields: string[]): Query;
    withOrFields(...orFields: string[]): Query;
    withSorts(...sorts: Sort[]): Query;
    withIncludedNavigations(...navigationNames: string[]): Query;
    withExcludedNavigations(...navigationNames: string[]): Query;
    withQueryParams(queryParams: any | string): Query;
    private convertQueryString(queryParams);
    refineByValue(navigationName: string, value: string, exclude?: boolean): Query;
    refineByRange(navigationName: string, low: number, high: number, exclude?: boolean): Query;
    restrictNavigation(restrictNavigation: RestrictNavigation): Query;
    skip(skip: number): Query;
    withPageSize(pageSize: number): Query;
    withMatchStrategy(matchStrategy: MatchStrategy): Query;
    withBiasing(biasing: Biasing): Query;
    enableWildcardSearch(): Query;
    disableAutocorrection(): Query;
    disableBinaryPayload(): Query;
    allowPrunedRefinements(): Query;
    build(): Request;
    private clearEmptyArrays(request);
}

import { Request } from '../models/request';
import { Results, Record } from '../models/response';
import { Query } from './query';
export interface RawRecord extends Record {
    _id: string;
    _u: string;
    _t: string;
    _snippet?: string;
}
export declare abstract class AbstractBridge {
    protected bridgeUrl: string;
    protected abstract augmentRequest(request: any): any;
    search(query: string | Query | Request, callback?: (Error?, Results?) => void): Promise<Results>;
    private extractRequest(query);
    private generateError(error, callback);
    private fireRequest(url, body, queryParams);
    private convertRecordFields(record);
}
export declare class CloudBridge extends AbstractBridge {
    private clientKey;
    private bridgeRefinementsUrl;
    private bridgeRefinementsSearchUrl;
    private bridgeClusterUrl;
    constructor(clientKey: string, customerId: string);
    protected augmentRequest(request: any): any;
}
export declare class BrowserBridge extends AbstractBridge {
    constructor(customerId: string);
    protected augmentRequest(request: any): any;
}

import { FluxCapacitor } from './index';
export declare class Pager {
    private flux;
    constructor(flux: FluxCapacitor);
    next(): Promise<void>;
    last(): Promise<void>;
    private paginate(forward, predicate);
    private hasNext;
    private hasPrevious;
    private step(add);
    private lastStep;
    private total;
}

import { Query } from '../core/query';
import { BrowserBridge } from '../core/bridge';
import { Results } from '../models/response';
import { SelectedValueRefinement, SelectedRangeRefinement } from '../models/request';
export declare class FluxCapacitor {
    private originalQuery;
    query: Query;
    bridge: BrowserBridge;
    results: Results;
    constructor(endpoint: string, config?: any);
    nextPage(): Promise<void>;
    lastPage(): Promise<void>;
    search(query?: string): Promise<void>;
    reset(): Promise<void>;
    refine(refinement: SelectedValueRefinement | SelectedRangeRefinement): Promise<void>;
    unrefine(refinement: SelectedValueRefinement | SelectedRangeRefinement): Promise<void>;
}

/// <reference path="../typings/index.d.ts" />
import './polyfills';
export * from './core/query';
export * from './core/bridge';
export * from './capacitor/index';
export * from './utils/converter';
export * from './models/request';
export * from './models/response';
